/*******************************************************************************
 *  Copyright (C) 2021 Shanghai Panchip Microelectronics Co., Ltd. 
 *  All rights reserved.
 * 
 *  @file:  list.h
 *  @brief: 
 *  @history: - V1.0, 2021-12-13
 *      
 *******************************************************************************/
#ifndef _LIST_H_
#define _LIST_H_


#define NULL ((void *)0)
#define LIST_POISON1  NULL
#define LIST_POISON2  NULL

//membertypeеλ
#define offsetof(type, member)  (size_t)(&((type*)0)->member)

//memberĵַȡtypeʼַ
#define container_of(ptr, type, member) ({          \
        const typeof(((type *)0)->member)*__mptr = (ptr);    \
    (type *)((char *)__mptr - offsetof(type, member)); })

//ṹ
struct list_head
{
    struct list_head *prev;
    struct list_head *next;
};
#define LIST_HEAD_INIT(name) { &(name), &(name) }

#define LIST_HEAD(name) \
    struct list_head name = LIST_HEAD_INIT(name)
    
static inline void INIT_LIST_HEAD(struct list_head *list)
{
    list->next = list;
    list->prev = list;
}

static inline void init_list_head(struct list_head *list)
{
    list->prev = list;
    list->next = list;
}

#ifndef CONFIG_DEBUG_LIST
static inline void __list_add(struct list_head *new,
         struct list_head *prev,
         struct list_head *next)
{
    next->prev = new;
    new->next = next;
    new->prev = prev;
    prev->next = new;
}
#else
extern void __list_add(struct list_head *new,
         struct list_head *prev,
         struct list_head *next);
#endif

//ͷ
/**
    * list_add - add a new entry
    * @new: new entry to be added
    * @head: list head to add it after
    *
    * Insert a new entry after the specified head.
    * This is good for implementing stacks.
 */
#ifndef CONFIG_DEBUG_LIST
static inline void list_add(struct list_head *new, struct list_head *head)
{
    __list_add(new, head, head->next);
}
#else
extern void list_add(struct list_head *new, struct list_head *head);
#endif

//β
static inline void list_add_tail(struct list_head *new, struct list_head *head)
{
    __list_add(new, head->prev, head);
}

static inline  void __list_del(struct list_head *prev, struct list_head *next)
{
    prev->next = next;
    next->prev = prev;
}

static inline void list_del(struct list_head *entry)
{
    __list_del(entry->prev, entry->next);
    entry->next = LIST_POISON1;
    entry->prev = LIST_POISON2;
}


static inline void __list_splice(struct list_head *list,
        struct list_head *head)
{
    struct list_head *first = list->next;
    struct list_head *last = list->prev;
    struct list_head *at = head->next;

    first->prev = head;
    head->next = first;

    last->next = at;
    at->prev = last;
}

/**
    * list_empty - tests whether a list is empty
    * @head: the list to test.
 */
static inline int list_empty(const struct list_head *head)
{
    return head->next == head;
}

/**
    * list_splice - join two lists
    * @list: the new list to add.
    * @head: the place to add it in the first list.
 */
static inline void list_splice(struct list_head *list, struct list_head *head)
{
    if (!list_empty(list))
     __list_splice(list, head);
}

/**
    * list_replace - replace old entry by new one
    * @old : the element to be replaced
    * @new : the new element to insert
    *
    * If @old was empty, it will be overwritten.
 */
static inline void list_replace(struct list_head *old,
    struct list_head *new)
{
    new->next = old->next;
    new->next->prev = new;
    new->prev = old->prev;
    new->prev->next = new;
}

static inline void list_replace_init(struct list_head *old,
     struct list_head *new)
{
    list_replace(old, new);
    INIT_LIST_HEAD(old);
}
/**
    * list_move - delete from one list and add as another's head
    * @list: the entry to move
    * @head: the head that will precede our entry
 */
static inline void list_move(struct list_head *list, struct list_head *head)
{
    __list_del(list->prev, list->next);
    list_add(list, head);
}

/**
    * list_move_tail - delete from one list and add as another's tail
    * @list: the entry to move
    * @head: the head that will follow our entry
 */
static inline void list_move_tail(struct list_head *list,
      struct list_head *head)
{
    __list_del(list->prev, list->next);
    list_add_tail(list, head);
}

#define list_entry(ptr, type, member) \
    container_of(ptr, type, member)

#define list_first_entry(ptr, type, member) \
    list_entry((ptr)->next, type, member)

#define list_for_each(pos, head) \
    for (pos = (head)->next; pos != (head); pos = pos->next)

/**
    * list_for_each_entry - iterate over list of given type
    * @pos: the type * to use as a loop cursor.
    * @head: the head for your list.
    * @member: the name of the list_struct within the struct.
 */
#define list_for_each_entry(pos, head, member)      \
 for (pos = list_entry((head)->next, typeof(*pos), member); \
      &pos->member != (head);  \
      pos = list_entry(pos->member.next, typeof(*pos), member))
/**
    * list_for_each_prev - iterate over a list backwards
    * @pos: the &struct list_head to use as a loop cursor.
    * @head: the head for your list.
 */
#define list_for_each_prev(pos, head) \
 for (pos = (head)->prev;  pos != (head); \
         pos = pos->prev)

#endif